﻿
open System.IO
open System.Text.RegularExpressions

let dirs = 
  [|
    @"C:\Users\steven\SkyDrive\Projects\AbsInf"
    @"C:\Users\steven\SkyDrive\Projects\HORSSyntax"
    @"C:\Users\steven\SkyDrive\Projects\Identifier"
    @"C:\Users\steven\SkyDrive\Projects\RecScheme"
    @"C:\Users\steven\SkyDrive\Projects\Utils"
  |]

let rec allFiles dirs =
    seq { for dir in dirs do yield! Directory.GetFiles dir; yield! allFiles (Directory.GetDirectories dir) }

let rgType = new Regex(@"type", RegexOptions.Compiled)
let rgModule = new Regex(@"module", RegexOptions.Compiled)
let rgFunction = new Regex(@"^let|member", RegexOptions.Compiled)

let count (rg: Regex) s =
    s |> rg.Matches |> fun x -> x.Count

type Statistics = {
        NumOfLines: int; NumOfFiles: int;
        NumOfChars: int; NumOfTypes: int;
        NumOfModules: int; NumOfFunctions: int;
    } 

let getStats =
    allFiles 
    >> Seq.filter (fun f -> f.EndsWith ".fs" || f.EndsWith ".fsx")
    >> Seq.fold (fun acc f -> 
                    let contents = f |> File.ReadLines
                    { NumOfLines = acc.NumOfLines + Seq.length contents; 
                      NumOfFiles = acc.NumOfFiles + 1;
                      NumOfChars = acc.NumOfChars + Seq.sumBy String.length contents;
                      NumOfTypes = acc.NumOfTypes + Seq.sumBy (count rgType) contents;
                      NumOfModules = acc.NumOfModules + Seq.sumBy (count rgModule) contents;
                      NumOfFunctions = acc.NumOfFunctions + Seq.sumBy (count rgFunction) contents; } 
                        ) { NumOfLines = 0; NumOfFiles = 0; 
                            NumOfChars = 0; NumOfTypes = 0; 
                            NumOfModules = 0; NumOfFunctions = 0 }