﻿namespace HORSSyntax
open HORSSyntax

type SynTransition =
  {
    State: SynState
    Symbol: SynIdent
    Succs: List<Int * SynState>
  }

type SynTAutomaton =
  {
    Transitions: List<SynTransition>
  }

type SynPAutomaton =
  {
    Priorities: List<SynState * Int>
    Transitions: List<SynTransition>
  }

[<CompilationRepresentation(CompilationRepresentationFlags.ModuleSuffix)>]
module SynAutomaton =

  module Parsers = 
  
    open Parsers
    
    let pIntStatePair : Parser<Int * SynState> =
      sparse {
        do! pLParen
        let! n = pInt
        do! pComma
        let! q = SynIdent.Parsers.pSynIdent
        do! pRParen
        return (n, q)
      }

    let pTransition : Parser<SynTransition> =
      sparse {
        let! q = SynIdent.Parsers.pSynIdent
        let! c = SynIdent.Parsers.pSynIdent
        do! pArrow
        let! s = pStar pIntStatePair
        return { State = q; Symbol = c; Succs = s}
      }

    let pTAutomaton : Parser<SynTAutomaton> =
      sparse {
        let! ts = pList1 pTransition pComma
        return { Transitions = ts }
      }    

    let pPAutomaton : Parser<SynPAutomaton> =
      sparse {
        do! pLParen
        let! ts = pList1 pTransition pComma
        do! pSemicolon
        let! ps = pList1 (pPair SynIdent.Parsers.pSynIdent pInt) pComma
        do! pRParen
        return { Transitions = ts; Priorities = ps }
      }   
