﻿namespace HORSSyntax
open HORSSyntax

[<StructuredFormatDisplay("{show}")>]
type SynTerm =
  | SynId of SynIdent
  | SynCase of Int32 * SynTerm * List<SynTerm>
  | SynApp of SynTerm * SynTerm

  member private x.show = x.ToString ()

  member s.ToStr (b: Bool) : String =
    let condParen b s = if b then "(" + s + ")" else s
    match s with
    | SynId x -> x.Str
    | SynApp (u, v) ->
       condParen b (u.ToStr false + " " + v.ToStr true)
    | SynCase (n, x, ts) ->
        let tsstr = List.map (fun (t: SynTerm) -> t.ToStr true) ts
        let ss    = sprintf "_case %d %s %s" n (x.ToStr true) (String.concat " " tsstr)
        condParen b ss

    override s.ToString () = s.ToStr false

[<CompilationRepresentation(CompilationRepresentationFlags.ModuleSuffix)>]
module SynTerm =
  
  let rec startPos (t: SynTerm) : String * Int * Int =
    match t with
    | SynId x -> x.Pos
    | SynApp (u, _) -> startPos u
    | SynCase (n, x, _) -> 
        let fn, row, col = startPos x
        let numDigits = (n.ToString ()).Length
        (fn, row, col - numDigits - 2 - 5)  // A rough guess
   
  module Parsers =
    
    open FParsec
    open Parsers
    
    let pTerm, pTerm' : Parser<SynTerm> * Ref<Parser<SynTerm>> = createParserForwardedToRef () 

    let pATerm : Parser<SynTerm> = 
      SynIdent.Parsers.pSynIdent      --> fun v -> SynId v
      <|> pLParen *> pTerm *> pRParen --> fun ((_,t),_) -> t
    do pTerm' :=
         star1 pATerm --> fun xs -> List.reduce (fun x y -> SynApp (x,y)) xs
  
  let parse (s: String) : SynTerm =
    Parsers.parse Parsers.pTerm s