﻿namespace HORSSyntax
open HORSSyntax

type SynState = SynIdent

type SynTType = 
  | SynTBase of SynState
  | SynTArrow of List<SynTType> * SynTType

type SynPType = 
  | SynPBase of SynState
  | SynPArrow of List<SynPType * Int> * SynPType

[<CompilationRepresentation(CompilationRepresentationFlags.ModuleSuffix)>]
module SynType =

  module Parsers = 

    open FParsec
    open Parsers
    
    let pAnd : Parser<Unit> = pstring @"/\" >>. preturn ()

    let pTop : Parser<List<SynTType>> = (pString "Top" <|> pString "T" ) >>. preturn []

    let pBase : Parser<SynState> = 
      SynIdent.Parsers.pSynIdent

    let pBaseTy : Parser<SynTType> =
      pBase --> SynTBase

    let pIType, pIType' : Parser<List<SynTType>> * Ref<Parser<List<SynTType>>> =
      createParserForwardedToRef ()

    let pType, pType' : Parser<SynTType> * Ref<Parser<SynTType>> = 
      createParserForwardedToRef () 

    let pAType : Parser<SynTType> = 
      pBase                           --> fun v -> SynTBase v
      <|> pLParen *> pType *> pRParen --> fun ((_,t),_) -> t

    do pType' :=
         sparse {
           let! ss = pIType
           do! pArrow
           let! t = pType
           return SynTArrow (ss, t)
         } <|> pBaseTy

    do pIType' := 
         pTop <|> pList1 pAType pAnd


    let pPTop : Parser<List<SynPType * Int>> = (pString "Top" <|> pString "T" ) >>. preturn []

    let pPBaseTy : Parser<SynPType> =
      pBase --> SynPBase

    let pPIType, pPIType' : Parser<List<SynPType * Int>> * Ref<Parser<List<SynPType * Int>>> =
      createParserForwardedToRef ()

    let pPType, pPType' : Parser<SynPType> * Ref<Parser<SynPType>> = 
      createParserForwardedToRef () 

    let pPAType : Parser<SynPType * Int> = 
      pPair pBase pInt      --> fun (q,m) -> SynPBase q, m
      <|> pPair pPType pInt --> fun (t,m) -> t, m

    do pPType' :=
         sparse {
           let! ss = pPIType
           do! pArrow
           let! t = pPType
           return SynPArrow (ss, t)
         } <|> pPBaseTy

    do pPIType' := 
         pPTop <|> pList1 pPAType pAnd


