﻿namespace ModelCheckApp

// Code by Tomas Petricek. Hosted at http://fssnip.net/6D

[<AutoOpen>]
module AsyncChoice =

  open System.Threading

  type Microsoft.FSharp.Control.Async with
    /// Takes several asynchronous workflows and returns 
    /// the result of the first workflow that successfuly completes
    static member Choice(workflows) = 
      Async.FromContinuations(fun (cont, _, _) ->
        let cts = new CancellationTokenSource()
        let completed = ref false
        let lockObj = new obj()
        let synchronized f = lock lockObj f
    
        /// Called when a result is available - the function uses locks
        /// to make sure that it calls the continuation only once
        let completeOnce res =
          let run =
            synchronized(fun () ->
              if completed.Value then false
              else completed := true; true)
          if run then cont res
    
        /// Workflow that will be started for each argument - run the 
        /// operation, cancel pending workflows and then return result
        let runWorkflow workflow = async {
          let! res = workflow
          cts.Cancel()
          completeOnce res }
    
        // Start all workflows using cancellation token
        for work in workflows do
          Async.Start(runWorkflow work, cts.Token) )
