﻿let bindir =
  (System.IO.Path.Combine(
     System.Environment.CurrentDirectory,
     System.IO.Path.GetDirectoryName(fsi.CommandLineArgs.[0])))

//do System.Environment.CurrentDirectory <- bindir

#I @"../../FSharpLibs/FParsec/"
#I @"../../Utils/bin/Debug/"
#I @"../../ModelCheckLib/bin/Debug"
#r "Utils.dll"
#r "ModelCheckLib.dll"
#r "FParsecCS.dll"
#r "FParsec.dll"

open FParsec

// Handy parsers
let ws cs = spaces cs
let str_ws s = ws >>. pstring s >>. ws
let arr cs = str_ws "->" cs

let parens p = between (str_ws "(") (str_ws ")") p
let sqrs   p = between (str_ws "[") (str_ws "]") p
let braces p = between (str_ws "{") (str_ws "}") p

let idchars = '_' :: ['a' .. 'z'] @ ['A' .. 'Z'] @ ['0' .. '9']

let ident (cs: CharStream<'a>) : Reply<String> =
  ((many1Chars (anyOf idchars)) .>> ws) cs

let id = ident

module THORSHORS =
  let simpletype = many1Chars (anyOf ['(';')';'-';'>';'o';' ']) .>> ws

  let typing = tuple2 (id .>> str_ws ":") simpletype

  let typings = many1 (typing .>> str_ws ";")

  let terminals = ws >>. str_ws "terminals" >>. braces typings

  let nonterminals = str_ws "nonterminals" >>. braces typings


  let rule = tuple3 (many1Chars (anyOf idchars) .>> ws)
                    (many (many1Chars (anyOf idchars) .>> ws) .>> str_ws "=")
                    (many1Chars (noneOf [';']) .>> ws)

  let rulelist = many1 (rule .>> str_ws ";")
  
  let rules = str_ws "rules" >>. braces rulelist

  let transition = tuple3 (id .>> str_ws ",") (id .>> str_ws "->") (manyChars (noneOf [';']) .>> ws)

  let transitionlist = many1 (transition .>> str_ws ";") 

  let transitions = str_ws "transitions" >>. braces transitionlist

//  let rejectOrAccept = (pstring "reject" <|> pstring "accept")
//
//  let acceptance = tuple2 (rejectOrAccept .>> str_ws ":") (many1 id)
//
//  let acceptancelist = many1 (acceptance .>> str_ws ";")
//
//  let acceptances = str_ws "acceptance" >>. braces acceptancelist

  let top = tuple4 terminals nonterminals rules transitions //acceptances

  let parse s =
    match run top s with
    | Success (result,_,_) -> result
    | Failure (errmsg,_,_) -> failwith errmsg
      


let prRule (nt,vars,rhs) =
  sprintf "%s %s -> %s." nt (String.concat " " vars) rhs

let getState (q,_,_) = q

let prTrans (q,a,qs) =
  sprintf "%s %s -> %s." q a qs

let prAcceptance (priority,qs) =
  let p = if priority = "accept" then "A" else "R"
  List.map (fun q -> sprintf "%s -> %s." q p) qs
  |> String.concat "\n"

let convert s =
  let _,_,r,a(*,p*) = THORSHORS.parse s
  in
//    sprintf "%%BEGING\n%s\n%%ENDG\n\n%%BEGINP\n%s\n%%ENDP\n\n%%BEGINA\n%s\n%%ENDA\n"
    sprintf "%%BEGING\n%s\n%%ENDG\n\n%%BEGINA\n%s\n%%ENDA\n"
      (List.map prRule r |> String.concat "\n" |> sprintf "%s")
      //(List.map prAcceptance p |> String.concat "\n" |> sprintf "%s")
      (List.map prTrans a |> String.concat "\n" |> sprintf "%s")

let processOne s = printfn "%s" s; IO.writeAll (System.IO.Path.ChangeExtension(s,".hrs")) (convert (IO.readAll s))

System.IO.Directory.GetFiles "/auto/users/robnea/dev/toolicfp2012/ModelCheckApp/examples/trecs/awt/"
|> Array.filter (fun p -> System.IO.Path.GetExtension(p) = ".rs")
|> Array.iter processOne

do if fsi.CommandLineArgs.Length >= 2 then
     do Array.iter processOne fsi.CommandLineArgs.[1..]


let eg = "terminals {
case : o -> o -> o -> o;
aI : o -> o;
bI : o -> o;
endI : o -> o;
b : o -> o;
a : o -> o;
end : o;
}

nonterminals {
S : o;
Empty : o;
Rev : o;
RevE : o;
Concat : o;
}




rules {
S = Rev Empty;
Empty x = x;
Rev x = case (endI (RevE x)) (aI (Rev (Concat a x))) (bI (Rev (Concat b x)));
RevE x = x end;
Concat x y z = x(y(z));
}


transitions{
q0,case -> q0 q0 q0;
q1,case -> q1 q1 q1;
q2,case -> q2 q2 q2;
q3,case -> q3 q3 q3;
qany,case -> qany qany qany;
q0,aI -> q0;
q0,bI -> q1;
q1,bI -> q1;
q1,aI -> qany;
q0,endI -> q2;
q1,endI -> q2;
q2,b -> q2;
q2,a -> q3;
q3,a -> q3;
q2,end -> ;
q3,end -> ;
qany,aI -> qany;
qany,bI -> qany;
qany,endI -> qany;
qany,a -> qany;
qany,b -> qany;
qany,end -> ;
}

"
  
  
