
open System.IO
open System.Text.RegularExpressions

let isRunningOnMono = System.Type.GetType ("Mono.Runtime") <> null


// Expects to be run in the directory where ModelCheckApp.sln is found

let rec findAllEgs d =
  let ds = Directory.GetDirectories d
  let fs = Directory.GetFiles d
           |> Array.filter (fun f -> Path.GetExtension f = ".hrs")
  Array.append fs (Array.collect findAllEgs ds)
  

let runMCApp args file =
  let p = new System.Diagnostics.Process()

  let mcapp = Path.Combine ("ModelCheckApp", "bin", "Debug", "ModelCheckApp.exe")
  let args = sprintf " --timing -1 --opts=order1 %s %s" args file
  if isRunningOnMono then
    p.StartInfo.FileName <- "mono"
    p.StartInfo.Arguments <- mcapp + args
  else
    p.StartInfo.FileName <- mcapp
    p.StartInfo.Arguments <- args
  p.StartInfo.RedirectStandardOutput <- true
  //p.StartInfo.RedirectStandardError  <- true
  p.StartInfo.UseShellExecute <- false
  
  let output = new System.Text.StringBuilder()
  p.OutputDataReceived.Add (fun e -> if e.Data <> null then output.AppendLine(e.Data) |> ignore)
  
  printf "Running '%s'..." (p.StartInfo.Arguments)
   
  p.Start() |> ignore
  p.BeginOutputReadLine()
  
  //let s = p.StandardOutput.ReadToEnd()
  //let t = p.StandardError.ReadToEnd()
  if p.WaitForExit(5000) then
    printfn "completed"
    (output.ToString(),"")
  else
    p.Kill()
    printfn "killed"
    ("Result: Timeout","Force kill")
  
  
let outputToSummary (out:string,err:string) =
  let r =
    if out.Contains("Result: Yes") then
      'Y'
    else if out.Contains("Result: No") then
      'N'
    else if out.Contains("Result: Timeout") then
      'T'
    else if out.Contains("Result: Error (Type environment not consistent)") then
      'I'
    else if out.Contains("Result: Error") then
      'E'
    else
      'X'
  let t =
    let m = Regex.Match(out, @"Model Checking time: (\d+)ms")
    if m.Success then
      m.Groups.[1].Value
    else
      "N/A"
      
  sprintf "%c,%s" r t
       
let expected s =
  let d = Path.GetDirectoryName s
  d.EndsWith("Yes")       
  
let runEgs egs args =
  //let dir = Path.Combine (System.Environment.CurrentDirectory, "examples", "trecs", "dwt")
  //let egs = findAllEgs dir
  let xs = Array.map (runMCApp args) egs
           |> Array.map outputToSummary
  
  
  Array.iter2 (fun (res:string) eg ->
                 if expected eg && (res.StartsWith "N") ||
                    not (expected eg) && (res.StartsWith "Y")
                 then
                    printfn "Wrong answer for '%s', got '%b', expected '%b'" eg (not (expected eg)) (expected eg)) xs egs
  
  Array.zip 
    (Array.map Path.GetFileNameWithoutExtension egs)
    xs
  //|> Array.map (fun (x,y) -> sprintf "%s%s%c" x (String.replicate (25 - x.Length) " ") y)
  |> Array.map (fun (x,y) -> sprintf "%s" y)
  |> String.concat "\n"
  |> fun s -> s + "\n"
  |> (fun s -> File.WriteAllText ("results.log",s))

      
if fsi.CommandLineArgs.Length >= 2 then
  let egs = findAllEgs fsi.CommandLineArgs.[1]
  runEgs egs (String.concat " " fsi.CommandLineArgs.[2..])
    

  

