﻿module HORSCParser

open FParsec

open RecScheme
open Parsing



type SynTermCase =
  | Sym of LocIdent
  | App of SynTermCase * SynTermCase
  | Case of SynTermCase * List<LocIdent * SynTermCase>

type SynRule = 
  { 
    nt: LocIdent 
    args: List<LocIdent> 
    rhs: SynTermCase 
  }

type SynHORSC = Syntax.Typings * List<SynRule>

let pCase = pstring "case"

let pSynCaseTerm, pSynCaseTerm' = createParserForwardedToRef ()
let pChoice, pChoice'           = createParserForwardedToRef ()

let pASynCaseTerm = 
  pCase   *> pLParen *> pSynCaseTerm *> pComma *> list pChoice pComma  *> pRParen --> fun (((((_, _), s), _), cs), _) -> Case (s, cs)
  <|> pIdentifier                                                   --> fun s -> Sym s
  <|> pLParen *> pSynCaseTerm *> pRParen                        --> fun ((_, t), _) -> t
  
do pSynCaseTerm' :=
    star1 pASynCaseTerm                   --> fun xs -> List.reduce (fun x y -> App (x,y)) xs

do pChoice' :=
    pLParen *> pIdent *> pComma *> pSynCaseTerm *> pRParen --> fun ((((_,b),_),t),_) -> b, t

let pSynCaseRule =
   pIdent *> star pIdent *> pLongArrow *> pSynCaseTerm   --> fun (((f,vs),_), r) -> { nt=f; args=vs; rhs=r } : SynRule

let pSynCaseRules = 
  pLBrace *> list pSynCaseRule pComma *> pRBrace --> fun ((_,rs),_) -> rs : List<SynRule>

let pHORSC =  pTypings *> pSynCaseRules


let synHORSCToWPMRS ((ts,rs): SynHORSC) : Syntax.ExtPMRS =

  let convertRule ({ nt = nt; args = args; rhs = rhs }: SynRule) : List<Syntax.ExtRule> =
    let rec aux (t: SynTermCase) : Syntax.ExtTerm * List<Syntax.ExtRule> =
      match t with
      | Sym  i      -> Syntax.Sym i, []
      | App  (u,v)  -> let l, ls = aux u
                       let r, rs = aux v
                       Syntax.App (l, r), ls @ rs
      | Case (u,cs) -> let l, ls = aux u
                       let f = Syntax.genFreshIdent "Case"
                       let bs,ss = List.unzip cs
                       let cs',rrs = List.map aux ss |> List.unzip
                       let rs' = [ for (b,c) in (List.zip bs cs') -> { Syntax.ExtRule.nt = f; Syntax.ExtRule.args = List.map Syntax.Sym (b::args); Syntax.ExtRule.rhs = c } ] :: ls :: rrs
                                 |> List.concat
                       List.fold (fun t i -> Syntax.App (t, Syntax.Sym i)) (Syntax.App (Syntax.Sym f, l)) args, rs'
    in
      let t, rs = aux rhs
      { nt = nt; args = List.map Syntax.Sym args; rhs = t } :: rs

  let rs' = { Syntax.ExtRules.rules = List.collect convertRule rs }

  Syntax.EPMRSNoNOrSt (ts, rs')
