namespace ModelCheckApp

module Version =

  let name  = "TravMC (AWT)"

  let major = "3"
  let minor = "0"
  let patch = "0"

  let revision = @"unknown"

  let string = sprintf "%s, version %s.%s.%s (revision %s)"
                        name major minor patch revision

module Options =
  open ModelCheckLib

  let verbose = ref false
  let timeout = ref -1
  let valtree = ref 0
  let runsemi = ref true
  let rundual = ref true
  let info = ref false
  let latex = ref false
  let maxrounds = ref -1
  
  let verboseFilter : Ref<Option<Set<String>>> = ref None

  let private setIntRef r v = r := int v

  let p = new NDesk.Options.OptionSet()
  Seq.iter (fun (s:string,d:string,a:string -> unit) -> ignore (p.Add(s,d,a)))
    [ 
      "version", "display versioning information",
        fun _ -> printfn "%s" Version.string

      "i|info", "print information about example in question, and exit",
        fun _ -> info := true

      "d|debug", "enable debug output",
        fun _ -> Debug.verbose := true
      
      "dfilter=", "apply a comma-separated {FILTER} to verbose output",
        fun v -> Debug.categories := v.Split(',') |> set |> Some
      
      "g|graph", "dump the final graph and other information",
        fun _ -> Cfg.printGraph := true
      
      "timeout=", sprintf "set a {TIMEOUT} in ms  (default off)",
        setIntRef timeout
      
      "1", "run just the semi-algorithm", fun _ -> if !runsemi then rundual := false else runsemi := true
      
      "2", "run just the dual semi-algorithm", fun _ -> if !rundual then runsemi := false else rundual := true
      
      "v", "set ModelCheckLib to verbose mode", fun _ -> Cfg.verbose := true
      
      "opts=", "Select optimisations using comma-separated list from\n" +
              "order1,weak,reifycache,trivial (default: all are on)",
              fun v -> let ss = v.Split(',') |> set
                       let opts = [ ("order1", Cfg.order1opt)
                                    //("canonical", Cfg.canonicalopt)
                                    //("nodereify", Cfg.nodeReifyOpt)
                                    ("reifycache", Cfg.reifyCacheOpt)
                                    //("prune", Cfg.pruneOpt)
                                    ("trivial", Cfg.trivialopt)
                                    //("badness", Cfg.badnessopt)
                                    //("filteropen", Cfg.filterOpen)
                                    //("leastopens", Cfg.leastopensopt)
                                    ("weak", Cfg.weakopt)
                                    //("externalgame", Cfg.extGameOpt)
                                    ]
                       List.iter (fun (s,r) -> r := Set.contains s ss) opts

      "valtree=", "print a prefix of the value tree after {VALTREE} parallel\n" +
                  "head reductions to the file 'valtree.gv'",
                  setIntRef valtree
          
      "rounds=", "maximum number of rounds to run before exiting", setIntRef Cfg.maxrounds
                  
      "timing", "enable printing of model checking time", fun _ -> Cfg.timing := true
                  
      "latex", "print the timing information as a latex table row", fun _ -> latex := true
                  
      "certificate", "display the found parity-consistent type environment", fun _ -> Cfg.certificate := true

      "h|?|help", "display this help!",
        fun _ -> printfn "%s" Version.string;
                 p.WriteOptionDescriptions(stdout);
                 exit 0
    ]
    
