﻿module Parser

open BasicTypes
open ModelCheckLib
open FParsec
open RecScheme
open Parsing

let pInstanceHORS  = pHORS  *> pAutomaton
let pInstanceWPMRS = pPMRS  *> pAutomaton
let pInstanceHORSC = HORSCParser.pHORSC *> pAutomaton

let parseHORSC (s: String) : PMRS * Automaton =
  let h, a = parse pInstanceHORSC s

  let p = HORSCParser.synHORSCToWPMRS h

  match p with
  | Syntax.EPMRS (ts, nts, rs, main) -> exit 1
  | Syntax.EPMRSNoNOrSt (ts, rs) ->
      let tidenv, nidenv, videnv as e = Classify.PMRSFromExtContext [] ts rs
      let p' = LambdaLift.appPMRS p
      let wpmrs = Identify.pmrs e p'
  
      let _, sidenv = Classify.AutomatonFromTransitions (Syntax.automatonTransitions a)
      let calA = Identify.automaton (tidenv, sidenv) a
      in
        wpmrs, calA

let parseHORS (s: String) : PMRS * Automaton =
  let rs, ts = parse pInstanceHORS s

  let tidenv, nidenv, videnv as e = Classify.HORSFromRules (Syntax.HORSrules rs)
  let g = Identify.hors e rs

  let _, sidenv = Classify.AutomatonFromTransitions (Syntax.automatonTransitions ts)
  let calA = Identify.automaton (tidenv, sidenv) ts

  let pmrs = PMRS.ofHORS g

  in
    pmrs, calA

let parseWPMRS (s: String) : PMRS * Automaton =
  let p, a = parse pInstanceWPMRS s

  match p with
  | Syntax.EPMRS (ts, nts, rs, main) -> exit 1
  | Syntax.EPMRSNoNOrSt (ts, rs) ->
      let tidenv, nidenv, videnv as e = Classify.PMRSFromExtContext [] ts rs
      let p' = LambdaLift.appPMRS p
      let wpmrs = Identify.pmrs e p'
  
      let _, sidenv = Classify.AutomatonFromTransitions (Syntax.automatonTransitions a)
      let calA = Identify.automaton (tidenv, sidenv) a
      in
        wpmrs, calA