﻿
#I @"../FSharpLibs/FParsec"
#r @"../Utils/bin/Debug/Utils.dll"
#r @"../RecScheme/bin/Debug/RecScheme.dll"
#r "FParsecCS.dll"
#r "FParsec.dll"

#load "Timing.fs"
#load "Cfg.fs"
#load "TvType.fs"
#load "IntType.fs"
#load "TermCase.fs"
#load "HORSC.fs"
#load "BasicTypes.fs"
#load "Subtyping.fs"
#load "RoseTree.fs"
#load "Derivation.fs"
#load "ITvMap.fs"
#load "TvMapNaive.fs"
#load "CalD.fs"
#load "WeakPMRS.fs"
#load "Algorithm.fs"
#load "TestParsers.fs"

open ModelCheckLib
open BasicTypes
open RecScheme
open RecScheme.Parsing
open Testing
open TestParsers

let q0 = parse pState "q0"
let q1 = parse pState "q1"

open IntType
let t1 = Union (new HashSet<_>([q0]))
let t2 = Union (new HashSet<_>([q0;q1]))
let t3 = Union (new HashSet<_>([q0;q0]))

let ho s = ho s |> PMRS.ofHORS

let g1 = ho ("{S       --> F b a,
               F 'x 'y --> 'x (F (F 'x) 'y)}")

let a1 = au "{q0 b --> q1,
              q1 b --> q1,
              q1 a --> }"

let g2 = ho ("{S     --> b a}")

let g3 = ho ("{S    --> F a,
               F 'x --> b 'x}")

let g4 = ho ("{S    --> F a,
               F 'x --> c 'x 'x}")

let a2 = au "{q0 c --> q1 q0,
              q0 a --> ,
              q1 a --> ,
              q2 a --> ,
              q0 b --> q0,
              q2 b --> q2,
              q2 d -->}"

let g5 = ho "{S  --> F1 T,
              F1 'k --> G1 'k,
              G1 'k --> 'k,
              T  --> t T}"

let a3 = au "{q0 t --> q0}"


let g6 = ho "{S    --> F b,
              F 'x --> c ('x y) ('x z)}"

let a4 = au "{q0 c --> q0 q0,
              q0 b --> q0,
              q0 y --> ,
              q1 z --> 
              }"

let g7 = ho "{
S  --> F Z U,
Z 'x1 'x2 --> 'x2,
U 'x1 'x2 --> 'x1 'x2,
F 'x1 'x2 --> c ('x1 b a) (F 'x2 (P 'x1 'x2)),
P 'x1 'x2 'x3 'x4 --> 'x1 'x3 ('x2 'x3 'x4)
}"

let a5 = au " 
{
s_even c --> b_end_good s_odd1,
s_odd1 c --> b_end_bad s_odd2,
s_odd2 c --> b_end_bad s_even,
b_end_good a --> ,
b_end_good b --> b_end_bad,
b_end_bad b --> b_end_good
}"

// flow
let g8 = ho "{
S  --> C1 Id,
C1 'id --> 'id Lam (C2 'id),
C2 'id 'unused --> 'id LamPrime C3,
C3 'x --> 'x end,
Lam 'x --> flow 'x,
LamPrime 'x --> 'x,
Id 'x 'k --> 'k 'x
}"

let a6 = au "
{
q0 end --> 
}"

let g9 = ho "{
S  --> F c,
F 'x --> a 'x (F (b 'x))
}"

let a7 = au "
{
q0 a --> q0 q0,
q0 b --> q1,
q1 b --> q1,
q0 c --> ,
q1 c --> 
}
"

(*
let b1 = Algorithm.check g1 a1
          = Yes (witness "[S:{q0}; F: {(q1 -> q1) -> Top -> q1, (Top -> q1) -> Top -> q1, ((q1 -> q0) ^ (q1 -> q1)) -> Top -> q0}]")

let b2 = Algorithm.check g2 a2
          = Yes (witness "[S:{q0}]")

let b3 = Algorithm.check g3 a2
          = Yes (witness "[S:{q0}; F: {q0 -> q0}]")

let b4 = Algorithm.check g4 a2
          = Yes (witness "[S:{q0}; F: {q0 ^ q1 -> q0}]")

let b5 = Algorithm.check g5 a3
          = Yes (witness "[S: {q0}; F1: {q0->q0}; G1: {q0->q0}; T: {q0}]")
         
let b6 = Algorithm.check g6 a4
          = No (trace "")
  *)    


(*
 * Dynamic/interactive testing aids
 *)

let m : CalD.TvMapT = TvMapNaive.empty ()

let init calR calA =
  Algorithm.init calR calA

let nextNodes m d =
  match CalD.openNodesBreadthHash m d with
  | CalD.NowCheck ns -> ns |> List.map ((CalD.proof << RoseTree.root))
  | CalD.Complete _  -> []

let run calR calA m d = List.iter (fun j -> Algorithm.expand calR calA (Map.empty,Map.empty) m j 1)
                                  (nextNodes m d); d

let curg, cura = (HORSC.PMRSToHORSC g1), a1

let d = init curg cura

let r () = run curg cura m d

let dump d m =
  IO.writeAll @"h:\winprogs\tool\ModelCheckApp\cald.gv" (CalD.toDot d m);
  IO.writeAll @"h:\winprogs\tool\ModelCheckApp\cald.tex" (CalD.toTex d);

