﻿module BasicTypes

open ModelCheckLib

[<StructuredFormatDisplayAttribute("{show}")>]
type GroundSym =
  | TG of Terminal
  | NG of NonTerminal

  member this.show = 
    match this with
    | TG x -> x.show
    | NG x -> x.show

[<StructuredFormatDisplayAttribute("{show}")>]
type Binding<'a,'b> =
  { term: 'a; ty: 'b }

  member this.show =
    sprintf "%A: %A" this.term this.ty

  override this.ToString() = this.show
  
type MutableBag<'a> = System.Collections.Generic.Dictionary<'a,Int>

type BaseTypeEnv = Map<Terminal, State> * Map<State, Terminal>

// TODO: This should be somewhere else
let symbolTermCaseToTex t = 
    let rec showTermBr (b: Bool) (t: TermCase) : String =
      let showApp x y = showTermBr false x + @" \; " + showTermBr true y
      match t with
      | TermCase.Sym s       -> sprintf "%A" s
      | TermCase.App (u, v)  -> if b then "(" + showApp u v + ")" else showApp u v
    in
      showTermBr false t

type Witness = Map<NonTerminal,Set<IntType>>

type NTypeEnv = Map<NonTerminal,IntTypePSet>
type VTypeEnv = Map<Variable,IntTypePSet>
type TTypeEnv = Map<Terminal,IntTypePSet>

type CounterExample = List<TermCase> * List<Int>

[<StructuredFormatDisplayAttribute("{show}")>]
type Result =
  | Yes   of Witness
  | No
  | Error of String

  member this.show =
    match this with
    | Yes   w ->
      sprintf "Gamma =\n" + 
        (Map.toList w 
         |> List.map (fun (f,ts) -> sprintf "%A:%s" f (List.toStringWithDelims "\n  " "\n  " "\n\n" (Seq.toList  ts)))
         |> String.concat "")
    | No           -> "No"
    | Error s      -> "Error: " + s