﻿
#I @"..\FSharpLibs\FParsec"
#r @"..\Utils\bin\Debug\Utils.dll"
#r @"..\RecScheme\bin\Debug\RecScheme.dll"
#r "FParsecCS.dll"
#r "FParsec.dll"

#load "TvType.fs"
#load "IntType.fs"
#load "TermCase.fs"
#load "HORSC.fs"
#load "BasicTypes.fs"
#load "RoseTree.fs"
#load "Derivation.fs"
#load "TvMapNaive.fs"
#load "CalD.fs"
#load "Algorithm.fs"
#load "TestParsers.fs"

open TvMapNaive
open BasicTypes
open RecScheme
open Testing
open TestParsers



let p1 = pm "
              {
                b1 : d,
                b2 : d,
                b3 : d,
                ok1 : o,
                ok2 : o,
                error : o
              }
              {
                Main --> F X,
                F b1 --> ok1,
                F b2 --> ok2,
                F b3 --> error,
                X --> b1,
                X --> b2
              }
              "

let g1 = HORSC.PMRSToHORSC p1

let a1 = au "{
                q0 ok1 --> ,
                q0 ok2 --> 
              }"

let p2 = pm "{
  b1 : d,
  b2 : d,
  ok : o,
  error : o
}
{
  Main --> F X X,
  F b1 b1 --> ok,
  F b1 b2 --> error,
  F b2 b1 --> error,
  F b2 b2 --> error,
  X --> b1
}"

let a2 = au "
{
  q0 ok --> 
}"

let p3 = pm "
{ 
  z: nat,
  nz: nat,

  nil: natlist,
  cons: natlist,

  nill: natlistt,
  conss: natlistt,

  true: bool,
  false: bool,

  br : o -> o -> o,
  error : o,
  ok : o
}
{
  Main --> Map Head S,
  Map 'f nill --> ok,
  Map 'f conss --> br ('f ListN) (Map 'f ListNN),
  Head nil --> error,
  Head cons --> ok,

  S --> ListNN,
  ListNN --> nill,
  ListNN --> conss,
  ListN --> cons,
  N --> z,
  N --> nz
}"

let g3 = HORSC.PMRSToHORSC p3

let a3 = au "
{
  q0 ok -->,
  q0 br --> q0 q0 
}"

let p4 = pm "
{
  a: o -> o, b: o -> o, e: o
}
{
  Main --> a S,
  S --> WordAB,
  WordAB --> a WordAB,
  WordAB --> WordB,
  WordB  --> b WordB,
  WordB  --> e
}"

let a4 = au "
{
  q0 b --> q0,
  q0 e --> ,
  q0 a --> q1,
  q1 a --> q1,
  q1 e --> 
}"

let p6 = pm "{
  c_zero: nat,
  c_succ: nat,

  c_nil: natlist,
  c_cons: natlist,

  error : nat,
  err : o,
  ok : o
}
{
  Main --> Check (Head ListN),

  Check error --> err,
  Check c_zero --> ok,
  Check c_succ --> ok,

  Head c_nil --> error,
  Head c_cons --> N,

  Nil --> c_nil,
  Cons c_zero c_nil  --> c_cons,
  Cons c_succ c_nil  --> c_cons,
  Cons c_zero c_cons --> c_cons,
  Cons c_succ c_cons --> c_cons,

  ListN --> Cons N Nil,
  ListN --> Cons N ListN,
  N --> Zero,
  N --> Succ N,

  Zero --> c_zero,
  Succ c_zero --> c_succ,
  Succ c_succ --> c_succ
}"
let a6 = au "{
  q0 ok -->
}"


let pflip = pm "{a: o, b: o -> o, br: o -> o -> o}
               {S --> F B,
                F 'x --> br ('x a) (G 'x),
                G 'x --> F 'x,
                B 'x --> b 'x}"
let gflip = HORSC.PMRSToHORSC pflip
let aflip = au "{q0 a -->,
                q0 b --> q0,
                q0 br --> q0 q0}"


let prepeat = pm "{a: o -> o, b: o -> o, c: o, br: o -> o -> o}
               {S --> G F,
                G 'g --> br ('g A) ('g B),
                F 'f --> 'f ('f c),
                A 'x --> a 'x,
                B 'x --> b 'x}"
let grepeat = HORSC.PMRSToHORSC prepeat
let arepeat = au "{q0 a --> q0,
                q0 b --> q0,
                q0 c -->,
                q0 br --> q0 q0}"

let ptrav = pm "{a: o -> o, b: o -> o, c: o, br: o -> o -> o}
               {S --> H a,
                H 'z --> F (g 'z),
                F 'phi --> 'phi ('phi (F h))
                }"
let gtrav = HORSC.PMRSToHORSC ptrav
let atrav = au "{q0 g --> q0 q0,
                 q0 h --> q0,
                 q0 a -->
                }"

(*
 * Dynamic/interactive testing aids
 *)

let m : CalD.TvMapT = TvMap.empty ()

let init calR calA =
  Algorithm.init calR calA

let btype g = HORSC.baseTypes g

let run calR calA b m d = List.iter (Algorithm.expand calR calA b m) (CalD.openNodesAll m d); d

let curg, cura, b = gtrav, atrav, (btype ptrav)

let d = init curg cura

let r () = run curg cura b m d

let dump d m =
  IO.writeAll @"h:\winprogs\toolcav\ModelCheckApp\cald.gv" (CalD.toDot d m);
  IO.writeAll @"h:\winprogs\toolcav\ModelCheckApp\cald.tex" (CalD.toTex d);
