﻿module Log

type Target = Stdout
            | Stderr
            | NewFile of String
            | AppendFile of String

type Level = Debug
           | Warn
           | Error
           | Fatal

let private target = ref stdout
let private level  = ref Error

let setLogLevel (l: Level) = level := l

let setTarget (t: Target) =
  (!target).Close()
  match t with
  | Stdout       -> target := stdout
  | Stderr       -> target := stderr
  | NewFile s    -> target := upcast System.IO.File.CreateText(s)
  | AppendFile s -> target := upcast System.IO.File.AppendText(s)

let close () = (!target).Close()

//let private output (fmt: Printf.TextWriterFormat<'a,unit>) : 'a = Printf.fprintfn !target fmt
let private output (s: String) : Unit = (!target).WriteLine(s)

let debug s =
  match !level with
  | Debug -> output s
  | Warn | Error | Fatal -> ()
  
let warn s =
  match !level with
  | Debug | Warn -> output s
  | Error | Fatal -> ()

let error s =
  match !level with
  | Debug | Warn | Error -> output s
  | Fatal -> ()
  
let fatal s =
  match !level with
  | Debug | Warn | Error | Fatal -> output s

let debugl (s: Lazy<String>) =
  match !level with
  | Debug -> output (s.Force())
  | Warn | Error | Fatal -> ()

let warnl (s: Lazy<String>) =
  match !level with
  | Debug | Warn  -> output (s.Force())
  | Error | Fatal -> ()

let errorl (s: Lazy<String>) =
  match !level with
  | Debug | Warn | Error -> output (s.Force())
  | Fatal -> ()

let fatall (s: Lazy<String>) =
  match !level with
  | Debug | Warn | Error | Fatal -> output (s.Force())

let debugf fmt = Printf.kprintf debug fmt

let warnf fmt  = Printf.kprintf warn fmt

let errorf fmt = Printf.kprintf error fmt

let fatalf fmt = Printf.kprintf fatal fmt