namespace ModelCheckLib
open ModelCheckLib
open Identifier
open HORSSyntax

[<StructuredFormatDisplayAttribute("{show}")>]
type Terminal = 
  { name: Ident }

  member this.show = this.name

[<StructuredFormatDisplayAttribute("{show}")>]
type Variable =
  { name: Ident }

  member this.show = this.name

[<StructuredFormatDisplayAttribute("{show}")>]
type NonTerminal =
  { name: Ident }

  member this.show = this.name

[<StructuredFormatDisplayAttribute("{show}")>]
type State =
  { name: Ident }

  member this.show = this.name

[<StructuredFormatDisplayAttribute("{show}")>]
type Symbol =
  | T of Terminal
  | N of NonTerminal
  | V of Variable

  member this.show = 
    match this with
    | T x -> x.show
    | N x -> x.show
    | V x -> x.show
    
[<CompilationRepresentation(CompilationRepresentationFlags.ModuleSuffix)>]
module Symbol =

//  let getIdent (s: Symbol) : Ident =
//    match s with
//    | T x -> x.name
//    | N x -> x.name
//    | V x -> x.name
 
  let reMkSym (s: Symbol) (i: SynIdent) : Symbol =
    let fn, p1, p2 = i.Pos
    let sp =
      Sourced { Filename = fn
                StartRow = p1
                StartCol = p2 }
    match s with
    | T x -> { Terminal.name = Ident.mkSameWithSrcs x.name sp } |> T
    | N x -> { NonTerminal.name = Ident.mkSameWithSrcs x.name sp } |> N
    | V x -> { Variable.name = Ident.mkSameWithSrcs x.name sp } |> V
 
 