﻿namespace ModelCheckLib
open ModelCheckLib
open HORSSyntax

// This type did contain a `Case' constructor for
// the ICFP'12 paper, but it has been removed for now.
[<StructuredFormatDisplayAttribute("{show}")>]
type TermCase =
  | Sym of Symbol
  | App of TermCase * TermCase

  member this.show = 
    let rec showExprBr (b: Bool) (t: TermCase) : String =
      let showApp x y = showExprBr false x + " " + showExprBr true y
      match t with
      | Sym s       -> sprintf "%A" s
      | App (u, v)  -> if b then "(" + showApp u v + ")" else showApp u v
    showExprBr false this

  override this.ToString() = this.show

[<CompilationRepresentation(CompilationRepresentationFlags.ModuleSuffix)>]
module TermCase =

  let ofSyntax (vmap: Classifier<Variable>) ((nmap,tmap,_): Classification) (t: SynTerm) : TermCase =
    let symMap sym = match nmap sym with
                     | None -> T (Option.get (tmap sym))
                     | Some nt -> N nt
    let rec convert s =
       match s with
        | SynId id -> Sym (match vmap id with
                           | Some sym -> V sym
                           | None -> symMap id)
        | SynApp (s,t) -> App (convert s, convert t)
        | SynCase (_,_,_) -> failwithf "Cannot yet handle input schemes with case"
                             
    convert t

  let long (t: TermCase) : Symbol * List<TermCase> =
    let rec long' acc u =
      match u with
      | Sym a      -> (a, acc)
      | App (u, v) -> long' (v::acc) u
    let s, ts = long' [] t
    (s, ts)

  let rec substitute (t: TermCase) (sub: Map<Variable,TermCase>) : TermCase =
    match t with
    | Sym (V x) when Map.containsKey x sub -> sub.[x]
    | Sym _ -> t
    | App (s,t) -> App (substitute s sub, substitute t sub)

  let isVariable t = match t with (Sym (V _)) -> true | _ -> false
  let isTerminal t = match t with (Sym (T _)) -> true | _ -> false
  let isNonTerminal t = match t with (Sym (N _)) -> true | _ -> false
  let isApp t = match t with (App _) -> true | _ -> false
  
  let rec size t =
    match t with
    | Sym _      -> 1
    | App (u, v) -> size u + size v

  let toDot (t: TermCase) : String =
    let rec loop t id =
      let edge id n = sprintf "node%s -> node%s;" id (id + n.ToString())
      match long t with
      | (T s), ts ->
          let decl = sprintf "node%s [label=\"%A\"];" id s
          let edges = List.map (edge id) [0 .. ts.Length-1]
          decl :: edges @ List.mapi (fun i t -> loop t (id + i.ToString())) ts
          |> String.concat "\n"
      | _ -> sprintf "node%s [label=\"%A\"];" id t
    let gr = loop t "0"
    sprintf "digraph {\n%s\n}\n" gr
    
  
  
  
  