﻿namespace ModelCheckLib

open System.Diagnostics

type Timer =
  {
    sw:    Stopwatch
    total: Float
    ts:    Ref<List<Float>>
  }


module Timing =
  
  let hits = ref 0
  let misses = ref 0

  let reifyTimer = Stopwatch()
  let reifyTimes : Ref<List<Float>> = ref []

  let empty (_: Unit) : Timer =
    {
      sw = Stopwatch();
      total = 0.0;
      ts = ref []
    }

  let start   (t: Timer) : Unit = t.sw.Restart()
  
  let stop    (t: Timer) : Unit =
    do t.sw.Stop()
    do t.ts := t.sw.Elapsed.TotalMilliseconds :: !t.ts

  let get (t: Timer) = !t.ts

  let getTotal (t: Timer) = List.sum (!t.ts)
  
  
  type RoundInfo =
    {
      treeSize : Int
      expansionTime : Int64
      terminationCheckTime : Int64
      maximalSubsetMin : Int
      maximalSubsetMax : Int
      bronKerboschTime : Int64
      reificationTime : Int64
      gameTime : Int64    
    }
  
  let store = new ResizeArray<RoundInfo>()
  
  let addNewCell () = store.Add { treeSize = 0; expansionTime = 0L; terminationCheckTime = 0L; maximalSubsetMin = Int.MaxValue; maximalSubsetMax = Int.MinValue; bronKerboschTime = 0L; reificationTime = 0L; gameTime = 0L }
  
  let getLast () = store.[store.Count-1]
    