﻿namespace ModelCheckLib
open ModelCheckLib


//[<CompilationRepresentation(CompilationRepresentationFlags.ModuleSuffix)>]
module TvType =

  [<StructuredFormatDisplayAttribute("{show}")>]
  type TyVar =
    { value: Int }

    member this.show  = sprintf "a%d" this.value
    member this.toTex = sprintf @"\alpha_{%d}" this.value
    override this.ToString() = this.show

  (* 
   * Types consisting of a list of type variables
   * with a single concrete state and priority in
   * the rightmost position of the form:
   *
   *    (a1 -> ... -> an -> q0, m)
   *)
  [<Inspect.PrimitivePrintAttribute>]
  [<StructuredFormatDisplayAttribute("{show}")>]
  type T =
    {
      stem:  List<TyVar>
      state: State
      priority: Int
    }

    member this.show =
      let ts = List.toStringWithDelims "" " -> " " -> " this.stem
      sprintf "(%s%A, %d)"
        (match this.stem with
         | [] -> ""
         | _  -> ts)
        this.state
        this.priority

    member this.toTex =
      let ts = List.map (sprintf "%A") this.stem
      sprintf "(%s%A, %d)"
        (match ts with
         | [] -> ""
         | _  -> (String.concat @" \to " ts) + @" \to ")
        this.state
        this.priority

    override this.ToString() = this.show

  let mk stem state p = { stem = stem; state = state; priority = p }

  let addLeft a theta = { theta with stem = a :: theta.stem }
