﻿
#I @"..\FSharpLibs\FParsec"
#r @"..\Utils\bin\Debug\Utils.dll"
#r @"..\RecScheme\bin\Debug\RecScheme.dll"
#r "FParsecCS.dll"
#r "FParsec.dll"

#load "TvType.fs"
#load "IntType.fs"
#load "BasicTypes.fs"
#load "RoseTree.fs"
#load "Derivation.fs"
#load "TvMap.fs"
#load "CalD.fs"
#load "TypeParser.fs"

//open FParsec

let states       = (List.map (toString >> (+) "q") [0..10])
let terminals    = ["a";"b";"c";"d";"e"]
let nonterminals = ["S";"F";"G";"H";"B"]
let variables    = ["x";"y";"z"]

let pt s = TypeParser.IntTypeParser.parse states s

let pt2 = TypeParser.IntTypeParser2.parse


let b1 = pt "q0"
                  =
                  IntType.Atom (Name.S (0u, "q0"))

let b2 = pt "q0 -> q1"
                  =
                  IntType.Arrow (
                   Set.singleton (IntType.Atom (Name.S (0u, "q0"))),
                   IntType.Atom (Name.S (1u, "q1")))

let b3 = pt "q10 -> q1"
                  =
                  IntType.Arrow (
                   Set.singleton (IntType.Atom (Name.S (10u, "q10"))),
                   IntType.Atom (Name.S (1u, "q1")))

let b4 = pt "q1 ^ q0 -> q2"
                  =
                  IntType.Arrow (
                   set [IntType.Atom (Name.S (1u, "q1"));
                        IntType.Atom (Name.S (0u, "q0"))],
                   IntType.Atom (Name.S (2u, "q")))

let b5 = pt "(q1 ^ q0) -> q1"
                  =
                  IntType.Arrow (
                   set [IntType.Atom (Name.S (1u, "q1"));
                        IntType.Atom (Name.S (0u, "q0"))],
                   IntType.Atom (Name.S (1u, "q1")))

let b6 = pt "(q0 -> q0) -> q0"
                  =
                  IntType.Arrow (
                   Set.singleton
                    (IntType.Arrow (
                      Set.singleton (IntType.Atom (Name.S (0u, "q0"))),
                      IntType.Atom (Name.S (0u, "q0")))),
                   IntType.Atom (Name.S (0u, "q0")))

let b7 = pt "q0 -> q0 -> q0"
                  =
                  pt "q0 -> (q0 -> q0)"

let pi = TypeParser.TvTypeParser.parse states

let c1 = pi "q0"
          = TvType.mkAtom (Name.S (0u, "q0"))

let c2 = pi "q10"
          = TvType.mkAtom (Name.S (10u, "q10"))

let c3 = pi "a1 -> q1"
          = TvType.mk [1u]
                      (Name.S (1u, "q1"))


let c4 = pi "a1 -> a0 -> a1 -> a2 -> q1"
          = TvType.mk [1u; 0u; 1u; 2u]
                      (Name.S (1u, "q1"))

let pm = TypeParser.TvMapParser.parse states

let d1 = pm "[{q0}; {}; {a0 -> q1;a1 -> q0}; {a2 -> a1 -> q0}]"


let result = [b1;b2;b3;b4;b5;b6;b7;c1;c2;c3;c4;]

let pj = TypeParser.JudgmentParser.parse states terminals nonterminals

let e1 = pj "|- S : q0"
let e2 = pj "|- F a b :a1 -> a0 -> a1 -> a2 -> q1"
let e3 = pj "|-S:q0"
let e4 = pj "|-S: q0"
let e5 = pj "|-S :q0"
let e6 = pj "|- F a1 (G b a2) a3 c d : q0"
let e7 = pj "|- a1 : q0"

let pterm = TypeParser.TermParser.parse terminals nonterminals variables

let f1 = pterm "a"
let f2 = pterm "x"
let f3 = pterm "F"
let f4 = pterm "F (x a) (y z b) G x x"

let prules = TypeParser.RuleSetParser.parse terminals nonterminals variables

let g1 = prules ("S     -> F B a,
                  F x y -> x (F (F x) y),
                  B x   -> b x")

let pauto = TypeParser.AutomatonParser.parse states terminals

let h1 = pauto "<
                {zero: nat, succ: nat -> nat, nil : natlist, cons:nat -> natlist -> natlist}
                { q0, q1, q2 }
                {
                  q0 c --> q1 q0,
                  q0 a  --> ,
                  q1 b --> q2,
                  q2 b --> q2,
                  q2 d --> 
                }
                q0
              >"