﻿module HashMap

let tryFind (x: 'k) (m: HashMap<'k,'v>) : Option<'v> =
  let mutable y: 'v = Unchecked.defaultof<'v>
  let b = m.TryGetValue (x, &y)
  if b then Some y else None

let find (x: 'k) (m: HashMap<'k,'v>) : 'v = m.[x]

let updateOrDefault (x: 'k) (f: 'v -> 'v) (d: 'v) (m: HashMap<'k,'v>) : Unit =
  match tryFind x m with
  | None   -> m.[x] <- d
  | Some v -> m.[x] <- f v

let map (f: 'k -> 'v -> 'w) (m: HashMap<'k, 'v>) : HashMap<'k, 'w> =
  let m' = HashMap(m.Count)
  for KeyValue(k,v) in m do
    m'.[k] <- f k v
  m'

let eqAsMaps (eq: 'v -> 'v -> Bool) (m1: HashMap<'k, 'v>) (m2: HashMap<'k, 'v>) : Bool =
  let compareKVs (k1, v1) (m: HashMap<_,_>) =
    match m.TryGetValue k1 with
    | true, v2 -> eq v1 v2
    | false, _ -> false
  Seq.forall (fun (KeyValue (k, v)) -> m1.ContainsKey(k)) m2
  && Seq.forall (fun (KeyValue (k, v)) -> compareKVs (k, v) m2) m1