﻿module Parsers

open FParsec
open CharParsers

exception ParseError of String

let (-->) (p: Parser<'a, 'u>) (f: 'a -> 'b) : Parser<'b, 'u> = 
  (p .>> spaces) |>> f
  
let ( *> ) (p: Parser<'a, 'u>) (q: Parser<'b, 'u>) : Parser<'a * 'b, 'u> =
  p .>>? spaces .>>.? q

let ( **> ) (p: Parser<'a, 'u>) (q: Parser<'b, 'u>) : Parser<'a * 'b, 'u> =
  p .>>.? q

let list (p: Parser<'a, 'u>) (sep: Parser<'b, 'u>) : Parser<List<'a>, 'u> =
  sepBy p (sep .>> spaces)

let pList (p: Parser<'a, 'u>) (sep: Parser<'b, 'u>) : Parser<List<'a>, 'u> =
  sepBy p (sep .>> spaces)

let pList1 (p: Parser<'a, 'u>) (sep: Parser<'b, 'u>) : Parser<List<'a>, 'u> =
  sepBy1 p (sep .>> spaces)

let star (p: Parser<'a, 'u>) : Parser<List<'a>, 'u> =
  many (p .>> spaces) 

let star1 (p: Parser<'a, 'u>) : Parser<List<'a>, 'u> =
  many1 (p .>> spaces)

let pStar (p: Parser<'a, 'u>) : Parser<List<'a>, 'u> =
  many (p .>> spaces) 

let pStar1 (p: Parser<'a, 'u>) : Parser<List<'a>, 'u> =
  many1 (p .>> spaces)

let run = CharParsers.run

type Parser<'a> = Parser<'a, Unit>

let pLParen    : Parser<Unit> = pchar '(' >>. preturn ()
let pRParen    : Parser<Unit> = pchar ')' >>. preturn ()
let pLBrace    : Parser<Unit> = pchar '{' >>. preturn ()
let pRBrace    : Parser<Unit> = pchar '}' >>. preturn ()
let pComma     : Parser<Unit> = pchar ',' >>. preturn ()
let pColon     : Parser<Unit> = pchar ':' >>. preturn ()
let pSemicolon : Parser<Unit> = pchar ';' >>. preturn ()
let pBStroke   : Parser<Unit> = pchar '\\' >>. preturn ()
let pEq        : Parser<Unit> = pchar '=' >>. preturn ()
let pBar       : Parser<Unit> = pchar '|' >>. preturn ()
let pUScore    : Parser<Unit> = pchar '_' >>. preturn ()
let pDot       : Parser<Unit> = pchar '.' >>. preturn ()
let pArrow     : Parser<Unit> = pstring "->" >>. preturn ()
let pTilde     : Parser<Unit> = pchar '~' >>. preturn ()
let pLAngle    : Parser<Unit> = pchar '<' >>. preturn ()
let pRAngle    : Parser<Unit> = pchar '>' >>. preturn ()
let pTurnstile : Parser<Unit> = pstring "|-" >>. preturn ()
let pFalse     : Parser<Unit> = pstring "false" >>. preturn ()

let pString = pstring
let pInt = FParsec.CharParsers.pint32

let pair (p1: Parser<'a>) (p2: Parser<'b>) : Parser<('a * 'b)> =
  pLParen *> p1 *> pComma *> p2 *> pRParen --> fun ((((_,r1),_),r2),_) -> r1, r2

let pPair (p1: Parser<'a>) (p2: Parser<'b>) : Parser<('a * 'b)> =
  pLParen *> p1 *> pComma *> p2 *> pRParen --> fun ((((_,r1),_),r2),_) -> r1, r2

let parse (p: Parser<'a>) (s: String) : 'a =
  match run (spaces >>. p .>> spaces) s with
  | Success (r, _, _) -> r
  | Failure (e, _, _) -> Debug.printc "Parsing" "%s" e
                         raise (ParseError e)
  
type SParseBuilder () =
  
  member x.Bind (y: Parser<'a>, f: 'a -> Parser<'b>) : Parser<'b> = 
    y .>>? spaces >>=? f

  member x.Return (y: 'a) : Parser<'a> =
    preturn y

  member x.Delay (f: Unit -> Parser<'a>) : Parser<'a> = fun str -> f () str

let sparse = new SParseBuilder()

let pInParens (p: Parser<'a>) : Parser<'a> =
  sparse {
    do! pLParen
    let! x = p
    do! pRParen
    return x
  }

let pInBraces (p: Parser<'a>) : Parser<'a> =
  sparse {
    do! pLBrace
    let! x = p
    do! pRBrace
    return x
  }
